// Jednostavan kalkulator u grafikom okruenju (Kalkulator).

import java.awt.*;
import java.awt.event.*;

public class Kalkulator extends Frame {

  String[] osn = {"2", "8", "10", "16"};      // Osnove brojevnih sistema.
  private int osnova = 10;                    // Trenutni brojevni sistema.

  private long x = 0, y = 0;                  // Registri kalkulatora.
  private Label[] reg = new Label [2];        // Prikaz registara.
  private static final int X = 0, Y = 1;      // Indeksi registara.

  private Panel ploRegistri() {               // Ploa za prikaz registara.
    Panel ploa = new Panel(new GridLayout(2, 1));
    Color[] podloge = {Color.ORANGE, Color.GRAY};
    Color[] slova   = {Color.BLUE, Color.WHITE};
    for (int i=0; i<2; i++) {
      reg[i] = new Label("0", Label.RIGHT);
      reg[i].setFont(new Font(null, Font.BOLD, 20));
      reg[i].setForeground(slova[i]);
      reg[i].setBackground(podloge[i]);
      ploa.add(reg[i]);
    }
    return ploa;
  }
 
  private void prikazi() {                    // Prikaz sadraja registara.
    reg[X].setText(Long.toString(x, osnova));
    reg[Y].setText(Long.toString(y, osnova));
  }

  private Button[] cifre = new Button [16];   // Dugmad s ciframa.
  private boolean noviBroj = true;            // Unosi se novi broj.

  private Panel ploCifre() {                  // Ploa za dugmad s ciframa.
    Panel ploa = new Panel(new GridLayout(4, 4));
    CifraAkcija oslukiva = new CifraAkcija();
    for (int i=0; i<16; i++) {
      ploa.add(cifre[i] = new Button(Integer.toString(i, 16)));
      cifre[i].addActionListener(oslukiva);
      cifre[i].setEnabled(i < osnova);
    }
    return ploa;
  }

  private class CifraAkcija implements ActionListener {   // Obrada priti-
    @Override                                             // saka na cifre.
    public void actionPerformed(ActionEvent dog) {
      String cif = ((Button)dog.getSource()).getLabel();
      String staro = reg[Y].getText();
      reg[Y].setText((noviBroj || staro.equals("0") ? "" : staro) + cif);
      noviBroj = false;
      try {
        y = Long.parseLong(reg[Y].getText(), osnova);
      } catch (NumberFormatException g) {
        reg[Y].setText("G R E  K A");
        noviBroj = true;
      }
    }
  }

  private Panel ploOsnove() { // Ploa za radio dugmad s osnovama brojevnih
    Panel ploa = new Panel(new GridLayout(osn.length, 1));     // sistema.
    ploa.setBackground(Color.YELLOW);
    CheckboxGroup grupa = new CheckboxGroup();
    OsnovaPromena oslukiva = new OsnovaPromena();
    for (String s: osn) {
       Checkbox radio = new Checkbox(s,grupa,Integer.parseInt(s)==osnova);
       ploa.add(radio);
       radio.addItemListener(oslukiva);
    }
    return ploa;
  }

  private class OsnovaPromena implements ItemListener { // Obrada promene
    @Override                                           // osnove brojevnog
    public void itemStateChanged(ItemEvent dog) {       // sistema.
      String ozn = ((Checkbox)dog.getSource()).getLabel();
      osnova = Integer.parseInt(ozn);
      for (int i=0; i<16; i++)
        cifre[i].setEnabled(i < osnova);
      prikazi();
    }
  }

  private String[] op = {"=", "0", "+=", "|=", "-=",  // Oznake operatora.
      "^=", "*=", "&=", "/=", "<<=", "%=", ">>="};

  private Panel ploOperatori() {          // Ploa za dugmad s operatorima.
    Panel ploa = new Panel(new GridLayout(0, 2));
    OperatorAkcija oslukiva = new OperatorAkcija();
    for (String s: op) {
      Button dugme = new Button(s);
      dugme.setForeground(Color.BLUE);
      dugme.setBackground(Color.YELLOW);
      dugme.addActionListener(oslukiva);
      ploa.add(dugme);
    }
    return ploa;
  }

  private class OperatorAkcija implements ActionListener { // Obrada priti-
    @Override                                              // saka na ope-
    public void actionPerformed(ActionEvent dog) {         // ratore.
      try {
        switch (((Button)dog.getSource()).getLabel().charAt(0)) {
          case '=': x   = y; break;    case '0': y   = 0; break;
          case '+': x  += y; break;    case '|': x  |= y; break;
          case '-': x  -= y; break;    case '&': x  &= y; break;
          case '*': x  *= y; break;    case '^': x  ^= y; break;
          case '/': x  /= y; break;    case '<': x <<= y; break;
          case '%': x  %= y; break;    case '>': x >>= y; break;
        }
        prikazi();
      } catch (ArithmeticException g) {
        reg[Y].setText("G R E  K A");
      }
      noviBroj = true;
    }
  }

  private boolean uniti = false;  // Da li unititi prozor pri zatvaranju?

  public Kalkulator(int x, int y) {           // Inicijalizacija:
    super("Kalkulator");                      // - naslov prozora,
    setBounds(x, y, 305, 238);                // - poloaj i dimenzije,
    setResizable(false);                      // - bez promene veliine,
    addWindowListener(new WindowAdapter() {   // - oslukiva:
      @Override
      public void windowClosing(WindowEvent dog)// ~ unitavanje prozora,
        { if (uniti) dispose(); else setVisible(false); }
    });
    add(ploRegistri(),  BorderLayout.NORTH);  // - popunjavanje prozora.
    add(ploCifre(),     BorderLayout.CENTER);
    add(ploOsnove(),    BorderLayout.WEST);
    add(ploOperatori(), BorderLayout.EAST);
  }

  public Kalkulator() { this(0, 0); }         // Podrazumevani konstruktor.

  public long rezultat() { return x; }        // Dohvatanje rezultata.

  public static void main(String[] varg) {    // Glavna metoda.
    Kalkulator kalk = new Kalkulator();
    kalk.setVisible(true);
    kalk.uniti = true;
  }
} // class Kalulator
